@@ -1,37 +1,33 @@
 package match::simple::XS;
 
-use 5.008000;
+use 5.012;
 use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.001';
+our $VERSION   = '0.002';
 
+require mro;
 require Carp;
 require XSLoader;
-XSLoader::load('match::simple::XS', $VERSION);
+XSLoader::load( 'match::simple::XS', $VERSION );
 
-sub _regexp { scalar($_[0] =~ $_[1]) }
+sub _regexp { scalar( $_[0] =~ $_[1] ) }
 
-if ($] >= 5.010)
-{
-	eval q[
-		no warnings;
-		use overload ();
-		sub _smartmatch {
-			overload::Method($_[1], "~~")
-				? !!( $_[0] ~~ $_[1] )
-				: Carp::croak("match::simple::XS cannot match")
-		}
-	];
-}
-else
-{
-	eval q[
-		sub _smartmatch {
-			Carp::croak("match::simple::XS cannot match")
-		}
-	];
+sub _overloaded_smartmatch {
+	my ( $obj ) = @_;
+
+	my @mro = @{ mro::get_linear_isa( ref $obj ) };
+	for my $class ( @mro ) {
+		my $name = "$class\::(~~";
+		my $overload = do {
+			no strict 'refs';
+			exists( &$name ) ? \&$name : undef;
+		};
+		return $overload if $overload;
+	}
+	
+	return;
 }
 
 1;
@@ -47,6 +43,8 @@ match::simple::XS - XS backend for match::simple
 
 =head1 DESCRIPTION
 
+The user-facing module is L<match::simple>.
+
 Nothing to see here; move along.
 
 =head1 SEE ALSO
@@ -66,7 +64,7 @@ The XS source code for L<Scalar::Util> was also very useful.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2014 by Toby Inkster.
+This software is copyright (c) 2014, 2022 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
